#!/bin/bash

CODE_VERSION_MMP=$(grep "^build\.version" src/main/resources/serviceconfig/olat.properties | sed "s/.*=//" | head -n 1)
CODE_VERSION_MM=${CODE_VERSION_MMP%.*}
GIT_REF=$CI_COMMIT_REF_NAME
GIT_REF_VERSION_MMU=${GIT_REF#*OpenOLAT_}
GIT_REF_VERSION_MM=${GIT_REF_VERSION_MMU%.uibk}

echo "The last 5 (un)merged tags:"
/usr/bin/git tag -l --sort creatordate|tail -n 5

echo "The last 5 (un)merged uibk tags:"
/usr/bin/git tag -l --sort creatordate|grep '\.uibk-'|tail -n 5

echo "The last 5 merged uibk tags:"
/usr/bin/git tag -l --merged HEAD --sort creatordate|grep '\.uibk-'|tail -n 5

echo "The last 5 merged uibk tags for this major, minor and patch version:"
/usr/bin/git tag -l --merged HEAD --sort creatordate|grep -F "${CODE_VERSION_MMP}.uibk-"|tail -n 5

LASTTAG=$(/usr/bin/git tag -l --merged HEAD --sort creatordate|grep -F "${CODE_VERSION_MMP}.uibk-"|tail -n 1)
LASTTAG_VERSION_MMPU=${LASTTAG#*OpenOLAT_}
LASTTAG_VERSION_MMP=${LASTTAG_VERSION_MMPU%.uibk*}
LASTTAG_VERSION_MM=${LASTTAG_VERSION_MMP%.*}

set -e

echo "Checking out the current head as the branch name we are building in:"
git checkout -B "$CI_BUILD_REF_NAME" "$CI_BUILD_REF"

echo "Testing that the major and minor version from olat.properties and the branch name match"
test "${CODE_VERSION_MM}" = "${GIT_REF_VERSION_MM}"

echo "Check if the olat.properties patch release is the same as in our last tag"
if [[ "$LASTTAG_VERSION_MMP" = "$CODE_VERSION_MMP" ]]
then
	echo "the olat.properties version is still the same as for our last release, count up the uibk build number"
	CURRENT_BUILDNR=${LASTTAG_VERSION_MMPU#*-}
	NEW_BUILDNR=$(printf "%03d" $((10#$CURRENT_BUILDNR + 1)))
else
	echo "the olat.properties version has changed so we start with 001 as the new buildnr"
	NEW_BUILDNR="001"
fi

echo "Test if this is a tagged build, we dont do a release in this case!"
if [[ ! -z "${CI_COMMIT_TAG}" ]]
then
	echo "This is a tagged build: ${CI_COMMIT_TAG}, dont do a release when there is already a tag set"
	exit 1
fi

echo "Test if the detached head commit ($CI_COMMIT_SHA) is equal to the release branch ($CI_COMMIT_REF_NAME)"
if git diff --exit-code $CI_COMMIT_SHA origin/$CI_COMMIT_REF_NAME
then
	echo "SHA and REF are equal so we can checkout the branch"
	git checkout $CI_COMMIT_REF_NAME
else
	echo "SHA and REF not equal stopping the release process!"
	exit 1
fi

ORIGIN_GIT_URL=$(git remote get-url origin)
ORIGIN_GIT_SERVER_URL=${ORIGIN_GIT_URL#*@}
ORIGIN_GIT_SERVER=${ORIGIN_GIT_SERVER_URL%%[:/]*}

ssh-keyscan $ORIGIN_GIT_SERVER > ~/.ssh/known_hosts

NEW_VERSION="${CODE_VERSION_MMP}.uibk-${NEW_BUILDNR}"
NEW_TAG="OpenOLAT_${NEW_VERSION}"

if /usr/bin/git tag -l | grep $NEW_TAG
then
	echo "New tagname already exists in the git repository: $NEW_TAG stopping the release process!"
	exit 1
fi

mvn -s .m2/settings.xml -B release:prepare -Darguments="-s .m2/settings.xml -DskipTests=true -DskipSeleniumTests=true -Dmaven.javadoc.skip=true" -DreleaseVersion=$NEW_VERSION -DdevelopmentVersion=${LASTTAG_VERSION_MM}-SNAPSHOT -Dtag=${NEW_TAG}
mvn -s .m2/settings.xml -B release:perform -Darguments="-s .m2/settings.xml -DskipTests=true -DskipSeleniumTests=true -Dmaven.javadoc.skip=true"
